   pro ebtel, ttime, heat, length, t, n, p, v, dem_tr, dem_cor,  $
              logtdem, f_ratio, rad_ratio, $
              classical=classical, dynamic=dynamic, dem_old=dem_old, $
              flux_nt=flux_nt, energy_nt=energy_nt, rtv=rtv
;
; NAME:  Enthalpy-Based Thermal Evolution of Loops (EBTEL)
;
; PURPOSE:
;   Compute the evolution of spatially-averaged (along the field) loop quantities 
;   using simplified equations.  The instantaneous differential emission measure of 
;   the transition region is also computed. 
;
; INPUTS:
;   ttime  = time array (s)
;   heat   = heating rate array (erg cm^-3 s^-1)   (direct heating only; no electron beam heating)
;              (the first element, heat(0), determines the initial static equilibrium)
;   length = loop half length (top of chromosphere to apex) (cm)
;
; OPTIONAL KEYWORD INPUTS:
;   classical = set to use the UNsaturated classical heat flux
;   dynamic   = set to use dynamical r1 and r2 (NOT recommended, specially when T > 10 MK)
;   dem_old   = set to use old technique of computing DEM(T) in the trans. reg.
;               (weighted average of demev, demcon, and demeq)
;   flux_nt   = energy flux array for nonthermal electrons impinging the chromosphere 
;               (input as a positive quantity; erg cm^-2 s^-1)
;   energy_nt = mean energy of the nonthermal electrons in keV (default is 50 keV)
;   rtv       = set to use Rosner, Tucker, Vaiana radiative loss function (Winebarger form)
;
; OUTPUTS:
;   t = temperature array corresponding to time (avg. over coronal section of loop)
;   n = electron number density array (cm^-3) (coronal avg.) 
;   p = pressure array (dyn cm^-2) (coronal avg.)
;   v = velocity array (cm s^-1) (r4 * velocity at base of corona)
;   dem_tr = differential emission measure of transition region, dem(time,T), both legs
;             (dem = n^2 * ds/dT  cm^-5 K^-1)
;             (Note:  dem_tr is not reliable when a nonthermal electron flux is used.)
;   dem_cor = differential emission measure of corona, dem(time,T), both legs
;             (Int{dem_cor+dem_tr dT} gives the total emission measure of a 
;             loop strand having a cross sectional area of 1 cm^2)
;   logtdem = logT array corresponding to dem_tr and dem_cor
;   f_ratio = ratio of heat flux to equilibrium heat flux
;             (ratio of heat flux to tr. reg. radiative loss rate)
;   rad_ratio = ratio of tr. reg. radiative loss rate from dem_tr and from r3*(coronal rate)
;
; CORRESPONDENCE WITH VARIABLES IN ASTROPHYSICAL JOURNAL ARTICLE:
;          (Klimchuk et al., 2008, ApJ, 682, 1351)
;   r1 = c_3
;   r2 = c_2
;   r3 = c_1
;   f, ff = F_0
;   f_eq, ff_eq = - R_tr
;   dem_eq = DEM_se
;
; USAGE:
;   To include the transition region DEM:
;      IDL> ebtel, ttime, heat, t, n, p, v, dem_tr, dem_cor, logtdem
;   To exclude the transition region DEM (faster):
;      IDL> ebtel, ttime, heat, t, n, p, v
;   To include a nonthermal electron energy flux:
;      IDL> ebtel, ttime, heat, t, n, p, v, flux_nt=flux_nt
;   To compute rad_ratio:
;      IDL> ebtel, ttime, heat, t, n, p, v, dem_tr, dem_cor, logtdem, f_ratio, rad_ratio
;      (Takes 25% more computing time.)
;
; INTENSITIES:
;   For observations in which temperature response function, G(T), has units of 
;   DN s^-1 pix^-1 cm^5 and the loop diameter, d, is larger than the pixel dimension, l_pix:
;
;      I_cor_perp = d/(2L)* Int{G(T)*dem_cor(T)*dT}
;      I_tr_perp = d/l_pix * Int{G(T)*dem_tr(T)*dT}
;      I_tr_parallel = Int{G(T)*dem_tr(T)*dT} ,
;
;   for lines-of-sight perpendicular and parallel to the loop axis.  I_tr_perp assumes that 
;   the transition region is thinner than l_pix.
;
; MISCELLANEOUS COMMENTS:
;   Runs much more quickly if the transition region DEM is not computed.
;   Speed can be increased by increasing the minimum DEM temperature from 10^4 to, say, 10^5 K 
;      or by decreasing the maximum DEM temperature from 10^8.5 to, say, 10^7.5 
;      (search on 450 and 451).
;   The equilibrium base heat flux coefficient of 2/7 is appropriate for uniform heating;
;      a coefficient of 4/7 is more appropriate for apex heating.
;   To have equal amounts of thermal and nonthermal heating:  flux_nt = heat*length.
;   It is desirable to have a low-level background heating during the cooling phase so that the 
;      coronal temperature does not drop below values at which the corona DEM is invalid.
;   r1 = c_3 = 0.7 gives more accurate coronal evolution than the original 0.5, especially in 
;      the late phase of cooling.  However, it produces excess DEM at the very hottest temperatures 
;      during impulsive events, and the transition region DEM is somewhat elevated.  We have 
;      therefore introduced r1_tr = 0.5, which provides a more accurate transition region DEM at 
;      the same time that r1 = 0.7 provides a more accurate radiative cooling.
;   v = (c_3/c_2)*(t_tr/t)*v_0 = (r1/r2)*(t_tr/t)*(v/r4) at temperature t_tr in the transition 
;      region, where t is the average coronal temperature.
;

; HISTORY:
;   01-jan-30, JAK, written.
;   01-jan-31, JAK, modified equations to account for differences
;              among apex, base, and loop average values.
;   05-may-04, JAK, added capability of dynamic r1 and r3.
;   05-may-05, JAK, added heat flux saturation and CLASSICAL keyword to ignore.
;   05-may-17, JAK, renamed time array ttime (to avoid strange conflict).
;   05-may-18, JAK, added dem option.
;   06-aug-25, JAK, improved calculation of dem.
;   06-nov-17, JAK, further improved calculation of dem (added dem_eq and dem_cor).
;   07-jan-11, JAK, added computation of tr. reg. radiative loss rate from dem_tr
;   07-jan-18, JAK, corrected computation of coronal dem.
;   07-jan-24, JAK, corrected computation of dynamic coefficients.
;   07-mar-19, JAK, made length an input variable; corrected r12 in rad_loss computation.
;   07-mar-23, JAK, added new method of computing transition region DEM(T).
;   07-mar-23, JAK, corrected factor of 2 in full loop coronal DEM(T).
;   07-jul-24, JAK, added dem_old keyword.
;   07-jul-24, JAK, increased allowable temperature range to logT=8.5.
;   07-jul-24, JAK, added thermal bremsstrahlung radiation (Cox & Tucker 1969).
;   07-aug-01, JAK, added notes about intensities.
;   07-aug-02, JAK, added nonthermal electron flux feature.
;   07-sep-28, JAK, set minimum temperature of 1.e4 K for coronal DEM calculation.
;   07-sep-28, JAK, added "no initial loop heating" message and return.
;   08-may-07, JAK, changed r1 (c_3) from 0.5 to 0.7.
;   08-sep-19, JAK, fixed a minor bug in dem_cor that makes time profiles look "ratty."
;   08-nov-04, JAK, added r1_tr to define temperature at top of transition region for dem_tr.
;   08-nov-04, JAK, add rtv keyword.
;   09-apr-30, JAK, corrected problem with 10^(-31) in RTV radiation law (result = 0).
;   09-jun-02, JAK, changed calculation of initial static equilibrium (no relaxation necessary).
;   09-jul-08, JAK, fixed problem with dem = 0 at final time step (set = previous time step).
;   10-jun-02, JAK, added warning about initial eq. being determined by first element of heat
;
; ---------------------------------------------------------------------

   ntot = n_elements(ttime)

; Physical constants
   k_b = 1.38e-16
   kappa_0 = 1.e-6      

; Ratio of base to apex temperature (c_3 in ApJ paper)
;      See miscellaneous comments above.
   r1 = 0.7

; Ratio of temperature at top of transition region to apex temperature, 
;      used only for calculation of transition region DEM, dem_tr 
;      See miscellaneous comments above.
   r1_tr = 0.5

;  Dynamic r1:
   if keyword_set(dynamic) then begin
      b0 = 0.783489
      b1 = -0.101746
      b2 = 0.00376242
      b3 = 7.81686e-005
      r1min = 0.2
      r1max = 0.7
   endif

; Ratio of average to apex temperature (c_2 in ApJ paper)
   r2 = 0.87

; Ratio of conductive to radiative losses in equilibrium (c_1 in ApJ paper)
   r3 = 4.0

; Dynamic r3:
   if keyword_set(dynamic) then begin
      d0 =  -0.0883701
      d1 =  -0.105824
      d2 =  0.688502
      d3 = -0.0467393
      r3max = 22.
      r3min = 1.
   endif

; Ratio of average to base velocity
   r4 = 1.0

; Invalid heat array
   if (heat(0) eq 0.) then begin
      print, '* * * * No initial loop heating * * * *'
      goto, jump99
   endif

   q = heat
   t = fltarr(ntot)
   n = fltarr(ntot)
   p = fltarr(ntot)
   v = fltarr(ntot)

; Radiation function temperature breaks

   if not keyword_set(rtv) then begin
;    Raymond-Klimchuk loss function
      kt0 = 1.e4
      kt1 = 9.3325e4
      kt2 = 4.67735e5
      kt3 = 1.51356e6
      kt4 = 3.54813e6
      kt5 = 7.94328e6
      kt6 = 4.28048e7

   endif else begin
;    RTV loss function
      kt0 = 10.^4.3
      kt1 = 10.^4.6
      kt2 = 10.^4.9
      kt3 = 10.^5.4
      kt4 = 10.^5.75
      kt5 = 10.^6.3
   endelse

; Thermal conduction parameters
   c1 = -2./7.*kappa_0  ;  conduction coefficient
   c_sat = -1.5*k_b^1.5/9.1e-28^0.5
   sat_limit = 1./6.

; Nonthermal electrons
   if not keyword_set(flux_nt) then flux_nt = ttime*0.
   flux_nt = -flux_nt

   if not keyword_set(energy_nt) then energy_nt = 50.   ; 50 keV
   j_nt = 6.241e8*flux_nt/energy_nt

   dlogt_cor = -alog10(r2)
   dj = fix(100*dlogt_cor)
   nj = 2*dj + 1

; DEM in transition region
   if n_params() gt 7 then begin

      logtdem = findgen(451)/100. + 4.
      tdem = 10.^logtdem
      root_tdem = tdem^0.5
      fourth_tdem = tdem^0.25
      dem_tr = fltarr(ntot,451)
      dem_cor = fltarr(ntot,451)   
      rad_dem = fltarr(451)
      demev = fltarr(ntot,451)
      demcon = demev
      demeq = demev
      rad_ratio = fltarr(ntot)

      f_ratio = fltarr(ntot)
      root_c2 = (kappa_0/(20.*k_b))^0.5/k_b    ; root_c2 to avoid overflow in dem_ev
      c3 = -5.*k_b
      c4 = (kappa_0/14.)^0.5/k_b

;    Radiation in transition region

      if not keyword_set(rtv) then begin
;       Raymond-Klimchuk loss function
         for i = 0, 450 do begin
            if (tdem(i) gt kt6) then  $
               rad_dem(i) = 1.96e-27*(tdem(i)^(1./2.)) else  $
            if (tdem(i) gt kt5) then  $
               rad_dem(i) = 5.4883e-16*(1./tdem(i)) else  $
            if (tdem(i) gt kt4) then  $
               rad_dem(i) = 3.4629e-25*(tdem(i)^(1./3.)) else  $
            if (tdem(i) gt kt3) then  $
               rad_dem(i) = 3.5300e-13*(tdem(i)^(-3./2.)) else  $
            if (tdem(i) gt kt2) then  $
               rad_dem(i) = 1.8957e-22 else  $
            if (tdem(i) ge kt1) then  $
               rad_dem(i) = 8.8669e-17*(1./tdem(i)) else $
            if (tdem(i) ge kt0) then  $
               rad_dem(i) = 1.0909e-31*tdem(i)^2  $
            else  $
               rad_dem(i) = 1.0  ;     appears in denominator of dem 
         endfor

      endif else begin
;       RTV loss function
         for i = 0, 450 do begin
            if (tdem(i) gt kt5) then  $
               rad_dem(i) = 10^(-17.73)/tdem(i)^(.666) else  $
            if (tdem(i) gt kt4) then  $
               rad_dem(i) = 10.^(-21.94) else  $
            if (tdem(i) gt kt3) then  $
               rad_dem(i) = 10.^(-10.4)/tdem(i)^2. else  $
            if (tdem(i) gt kt2) then  $
               rad_dem(i) = 10.^(-21.2) else  $
            if (tdem(i) ge kt1) then  $
               rad_dem(i) = 10^(-31.0)*tdem(i)^2. else $
            if (tdem(i) ge kt0) then  $
               rad_dem(i) = 10^(-21.85)  $
            else  $
            rad_dem(i) = 1.0  ;     appears in denominator of dem 
         endfor
      endelse

      root_rad_dem = rad_dem^0.5

   endif


; ---------------------------
; Initial static equilibrium
; ---------------------------

   if keyword_set(dynamic) then  $
      print, '* * * Using fixed r3 (c1) for initial static equilibrium * * *'

   tt = r2*(3.5*r3/(1. + r3)*length*length*q(0)/kappa_0)^(2./7.)

; Radiation

   if not keyword_set(rtv) then begin
;    Raymond-Klimchuk loss function
      if (tt gt kt6) then rad = 1.96e-27*sqrt(tt) else $
      if (tt gt kt5) then rad = 5.4883e-16/tt else $
      if (tt gt kt4) then rad = 3.4629e-25*(tt^(0.3333333)) else $
      if (tt gt kt3) then rad = 3.5300e-13/(tt^(1.5)) else $
      if (tt gt kt2) then rad = 1.8957e-22 else $
      if (tt gt kt1) then rad = 8.8669e-17/tt else $
      if (tt gt kt0) then rad = 1.0909e-31*tt*tt $
      else                rad = 0.0

   endif else begin
;    RTV loss function
      if (tt gt kt5) then rad = 10^(-17.73)/tt^(.666) else $
      if (tt gt kt4) then rad = 10.^(-21.94) else $
      if (tt gt kt3) then rad = 10.^(-10.4)/tt^2. else $
      if (tt gt kt2) then rad = 10.^(-21.2) else $
      if (tt gt kt1) then rad = 10^(-31.0)*tt^2. else $
      if (tt gt kt0) then rad = 10^(-21.85) $
      else                rad = 0.0
   endelse

   nn = (q(0)/((1. + r3)*rad))^0.5
   pp = 2.*k_b*nn*tt

   print, ' '
   print, 'Model parameters'
   print, '  r1 = ', r1
   print, '  r2 = ', r2
   print, '  r3 = ', r3
   print, ' '
   print, 'Initial static equilibrium'
   print, '  L = ', length
   print, '  Q = ', q(0)
   print, '  T = ', tt
   print, '  P = ', pp
   print, '  n = ', nn
   print, '***** Determined by first element of heat input array *****'

   t(0) = tt
   n(0) = nn
   p(0) = pp
   v(0) = 0.

; Scaling law values

   goto, jump1
   lambda_0 = 1.78e-19                               ;  lambda = lambda_0*T^bb
   bb = -0.5
   q_0 = heat(0)                                     ;  heating rate (erg cm^-3 s^-1)
   t_0 = r2*(3.5/kappa_0*q_0)^(2./7.)*length^(4./7.) ;  temperature (K) (avg., not apex)
   p_0 = r2^(-7./4.)*(8./7.*kappa_0/lambda_0)^0.5*k_b       $
         *t_0^((11.-2.*bb)/4.)/length                ;  total pressure (dyn cm^-2)
   n_0 = 0.5*p_0/(t_0*k_b)                           ;  electron number density (cm^-3)
   v_0 = 0.                                          ;  velocity

   print, ' '
   print, 'Scaling law values'
   print, '  T = ', t_0
   print, '  P = ', p_0
   print, '  n = ', n_0
jump1:


; ----------------------
; Time-dependent heating
; ----------------------

; Loop over time steps
   for i = 0, ntot-2 do begin

      dt = ttime(i+1) - ttime(i)

; Dynamic r1 and r3
      if keyword_set(dynamic) then begin
         t6 = t(i)/1.e6/r2
         r1 = b0 + b1*t6 + b2*t6^2 + b3*t6^3
         r1 = min([r1, r1max])
         r1 = max([r1, r1min])
         r3 = d0 + d1*t6 + d2*t6^2 + d3*t6^3
         r3 = min([r3, r3max])
         r3 = max([r3, r3min])
      endif

      r12 = r1/r2
      r12_tr = r1_tr/r2

; Thermal conduction flux at base

      f_cl = c1*(t(i)/r2)^3.5/length

      if keyword_set(classical) then begin
         f = f_cl
      endif else begin
         f_sat = sat_limit*c_sat*n(i)*t(i)^1.5
         f = -f_cl*f_sat/(f_cl*f_cl + f_sat*f_sat)^0.5
      endelse

; Radiation

      if not keyword_set(rtv) then begin
;       Raymond-Klimchuk loss function
         if (t(i) gt kt6) then rad = 1.96e-27*sqrt(t(i)) else $
         if (t(i) gt kt5) then rad = 5.4883e-16/t(i) else $
         if (t(i) gt kt4) then rad = 3.4629e-25*(t(i)^(0.3333333)) else $
         if (t(i) gt kt3) then rad = 3.5300e-13/(t(i)^(1.5)) else $
         if (t(i) gt kt2) then rad = 1.8957e-22 else $
         if (t(i) gt kt1) then rad = 8.8669e-17/t(i) else $
         if (t(i) gt kt0) then rad = 1.0909e-31*t(i)*t(i) $
         else                  rad = 0.0

      endif else begin
;       RTV loss function
         if (t(i) gt kt5) then rad = 10^(-17.73)/t(i)^(.666) else $
         if (t(i) gt kt4) then rad = 10.^(-21.94) else $
         if (t(i) gt kt3) then rad = 10.^(-10.4)/t(i)^2. else $
         if (t(i) gt kt2) then rad = 10.^(-21.2) else $
         if (t(i) gt kt1) then rad = 10^(-31.0)*t(i)^2. else $
         if (t(i) gt kt0) then rad = 10^(-21.85) $
         else                rad = 0.0
      endelse

; Equilibrium thermal conduction flux at base (-R_tr in ApJ paper)
      f_eq = -r3*n(i)*n(i)*rad*length

;      pv = 0.4*(f_eq - f)
      pv = 0.4*(f_eq - f - flux_nt(i))
;      dn = pv*0.5/(r12*k_b*t(i)*length)*dt
      dn = (pv*0.5/(r12*k_b*t(i)*length) + j_nt(i)/length)*dt
      n(i+1) = n(i) + dn

;      dp = 2./3.*(q(i) + (1. + 1./r3)*f_eq/length)*dt
      dp = 2./3.*(q(i) + (1. + 1./r3)*f_eq/length    $
           - (1. - 1.5*k_b*t(i)/energy_nt)*flux_nt(i)/length)*dt
      p(i+1) = p(i) + dp

      t(i+1) = p(i+1)/(n(i+1)*2.*k_b)

      v(i+1) = pv/p(i+1)

; Differential emission measure 
      if n_params() gt 7 then begin

;   Transition Region
         if (r12_tr*t(i) gt tdem(450)) then begin
            print, 'Transition region temperature outside DEM range'
            return
         endif
 
         if (f ne f_eq) then        $
            cf = f*f_eq/(f - f_eq)  $
         else                       $
            cf = 1.e10*f

         for j = 0, 450 do begin
            if (tdem(j) lt r12_tr*t(i)) then begin
;
            if not keyword_set(dem_old) then begin

;    New method
               aaa = kappa_0*tdem(j)^1.5
               bbb = -5.*k_b*n(i)*v(i)
               p2kt2 = (p(i)/(2.*k_b*tdem(j)))^2
               ccc = -p2kt2*rad_dem(j)
               dtds1 = (-bbb + (bbb*bbb - 4.*aaa*ccc)^0.5)/(2.*aaa)
               dtds2 = (-bbb - (bbb*bbb - 4.*aaa*ccc)^0.5)/(2.*aaa)
               dtds = max(dtds1, dtds2)
               dem_tr(i,j) = 2.*p2kt2/dtds  ; factor 2 for both legs

            endif else begin

;    Old method   
;           approximation to tr. reg. dem when evaporation dominates     
               dem_ev = (root_c2/n(i))*(root_c2*p(i)*p(i)  $
                        /root_tdem(j))/v(i)
;               vabs = abs(v(i))
;               vmin = max([vabs, abs(f_eq/(2.5*pv))])
;               dem_ev = (root_c2/n(i))*(root_c2*p(i)*p(i)  $
;                        /root_tdem(j))/vmin*v(i)/vabs

;           approximation to tr. reg. dem when condensation dominates
               dem_con = c3*n(i)*v(i)/rad_dem(j)

;           approximation to tr. reg. dem under equilibrium conditions
               dem_eq = c4*p(i)/(root_rad_dem(j)*fourth_tdem(j))

               dem_tr(i,j) = 2.*(f*dem_ev + cf*dem_eq - f_eq*dem_con)   $
                             /(f + cf - f_eq)              ; factor 2 for both legs

               demev(i,j) = 2.*dem_ev
               demcon(i,j) = 2.*dem_con
               demeq(i,j) = 2.*dem_eq
            endelse

               f_ratio(i) = f/f_eq

            endif

         endfor

         ss = where(dem_tr(i,*) lt 0., nneg)
         if (nneg gt 0) then begin
            print, ' '
            print, '***** Negative DEM;   i = ', i
            print, ' '
            if (i ne 0) then begin
                 for j = 0, 450 do dem_tr(i,j) = dem_tr(i-1,j)
            endif else begin
                 dem_tr(0,*) = 0.  ; to avoid problems at start of run with saturated heat flux
            endelse
         endif

;   Corona   (EM distributed uniformly over temperture interval [t_min, t_max])
         t_max = max([t(i)/r2, 1.1e4])
         t_min = max([t(i)*(2. - 1/r2), 1.e4])
         j_max = fix((alog10(t_max) - 4.0)*100)
         j_min = fix((alog10(t_min) - 4.0)*100)
  
         em = 2.*n(i)*n(i)*length            ; factor of 2 for both legs

;         dem0 = em/(t_max - t_min)
         delta_t = 10.^4*(10.^((j_max+0.5)/100.)    $
            - 10.^((j_min-0.5)/100.))
         dem0 = em/delta_t

         for j = j_min, j_max do   $
            dem_cor(i,j) = dem0


;   Transition region radiation losses based on DEM
         if n_params() gt 11 then begin
         rad_loss = 0.

         for j = 0, 450 do begin
            if (tdem(j) lt r12_tr*t(i)) then $
               rad_loss = rad_loss + dem_tr(i,j)*rad_dem(j)*tdem(j)*0.01*2.3  ; 2.3=ln(10)

         endfor

         rad_ratio(i) = -rad_loss/f_eq
         endif

      endif

   endfor

   v = r4*v

   dem_tr(ntot-1,*) = dem_tr(ntot-2,*)
   dem_cor(ntot-1,*) = dem_cor(ntot-2,*)

jump99:

   return
   end


